from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.cache import cache
from django.template.loader import render_to_string
from django.test import TestCase
from django.urls import reverse

from relatives.utils import object_edit_link, object_link

from .models import (
    Actor,
    Book,
    Eater,
    Image,
    Journal,
    Movie,
    NotInAdmin,
    Pet,
    Pirate,
    Sailor,
    Shape,
    Ship,
    Something,
)


class ObjectEditLinkTest(TestCase):
    def test_default(self):
        ship = Ship.objects.create(id=1, name="Star of India")
        self.assertEqual(
            object_edit_link()(ship),
            '<a href="/adm/tests/ship/1/change/">Star of India</a>',
        )
        pirate = Pirate.objects.create(id=1, name="Lowell Taylor")
        self.assertEqual(object_edit_link()(pirate), "Lowell Taylor")

    def test_custom_edit_text(self):
        ship = Ship.objects.create(id=1, name="Star of India")
        self.assertEqual(
            object_edit_link("Go There")(ship),
            '<a href="/adm/tests/ship/1/change/">Go There</a>',
        )

    def test_default_blank_text(self):
        pirate = Pirate.objects.create(id=1, name="Lowell Taylor")
        self.assertEqual(object_edit_link("Edit")(pirate), "")

    def test_custom_edit_and_blank_text(self):
        ship = Ship.objects.create(id=1, name="Star of India")
        pirate = Pirate.objects.create(id=1, name="Lowell Taylor")
        self.assertEqual(
            object_edit_link("Go There", "N/A")(ship),
            '<a href="/adm/tests/ship/1/change/">Go There</a>',
        )
        self.assertEqual(object_edit_link("Go There", "N/A")(pirate), "N/A")


class ObjectLinkTest(TestCase):
    def test_no_admin_url(self):
        pirate = Pirate.objects.create(id=1, name="Lowell Taylor")
        self.assertEqual(object_link(pirate), "Lowell Taylor")

    def test_with_primary_key(self):
        ship = Ship.objects.create(id=1, name="Star of India")
        self.assertEqual(
            object_link(ship),
            '<a href="/adm/tests/ship/1/change/">Star of India</a>',
        )

    def test_no_primary_key(self):
        ship = Ship(name="Star of India")
        self.assertEqual(object_link(ship), "Star of India")


class TemplateFilterTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_superuser("u", "u@example.com", "pass")

    def login(self):
        self.client.login(username=self.user.username, password="pass")

    def test_foreign_key(self):
        self.login()
        ship = Ship.objects.create(id=1, name="Star of India")
        sailor = Sailor.objects.create(name="John Ford", ship=ship)
        response = self.client.get(
            reverse("admin:tests_sailor_change", args=[sailor.id])
        )
        self.assertIn(
            b'<a href="/adm/tests/ship/1/change/">Star of India</a>', response.content
        )

    def test_property_and_modeladmin_method(self):
        self.login()
        circle = Shape.objects.create(id=1, name="Circle")
        response = self.client.get(
            reverse("admin:tests_shape_change", args=[circle.id])
        )
        self.assertIn(b"<p>Circle</p>", response.content)
        self.assertIn(b"<p>circle</p>", response.content)
        self.assertIn(b"<p>CIRCLE</p>", response.content)

    def test_no_foreign_key(self):
        self.login()
        ship = Ship.objects.create(id=1, name="Star of India")
        response = self.client.get(reverse("admin:tests_ship_change", args=[ship.id]))
        self.assertIn(b"<p>Star of India</p>", response.content)

    def test_foreign_key_without_admin_url(self):
        self.login()
        pirate = Pirate.objects.create(id=1, name="Kristi Bell")
        pet = Pet.objects.create(owner=pirate)
        response = self.client.get(reverse("admin:tests_pet_change", args=[pet.id]))
        self.assertIn(b"Kristi Bell", response.content)
        self.assertNotIn(b"Kristi Bell</a>", response.content)
        self.assertNotIn(b"Kristi Bell</option>", response.content)

    def test_nullable_foreign_key(self):
        self.login()
        sailor = Sailor.objects.create(name="John Ford")
        response = self.client.get(
            reverse("admin:tests_sailor_change", args=[sailor.id])
        )
        self.assertIn(b"<p>-</p>", response.content)

    def test_deleted_foreign_key(self):
        self.login()
        ship = Ship.objects.create(id=1, name="Star of India")
        sailor = Sailor.objects.create(name="John Ford", ship=ship)
        ship.delete()  # Sailor won't get deleted
        response = self.client.get(
            reverse("admin:tests_sailor_change", args=[sailor.id])
        )
        self.assertIn(b"<p>-</p>", response.content)

    def test_add_form_for_non_nullable_fk(self):
        self.login()
        response = self.client.get(reverse("admin:tests_pet_add"))
        self.assertIn(b"<p>-</p>", response.content)

    def test_generic_foreign_key_present(self):
        book = Book.objects.create(name="Django")
        image = Image.objects.create(content_object=book)
        self.login()
        response = self.client.get(reverse("admin:tests_image_change", args=[image.id]))
        self.assertIn(f"/adm/tests/book/{book.id}", response.rendered_content)
        self.assertIn(f"Book object ({book.id})", response.rendered_content)
        self.assertNotIn("<p>None</p>", response.rendered_content)

    def test_generic_foreign_key_not_present(self):
        image = Image.objects.create()
        self.login()
        response = self.client.get(reverse("admin:tests_image_change", args=[image.id]))
        self.assertIn("<p>None</p>", response.rendered_content)
        self.assertNotIn("Book object", response.rendered_content)


class RelatedObjectsTagTest(TestCase):
    def test_foreign_keys(self):
        ship = Ship.objects.create(id=1, name="Star of India")
        body = render_to_string("related_objects_fk_test.html", {"obj": ship})
        self.assertEqual(
            body.strip(),
            '<a href="/adm/tests/sailor/?ship__id__exact=1">Sailors</a>',
        )

    def test_two_foreign_keys(self):
        eater = Eater.objects.create(id=1, name="Cheryl")
        body = render_to_string("related_objects_fk_test.html", {"obj": eater})
        self.assertEqual(
            body.strip(),
            '<a href="/adm/tests/meal/?prepared__id__exact=1">'
            "Meals prepared</a>"
            '<a href="/adm/tests/meal/?reviewed__id__exact=1">'
            "Meals reviewed</a>",
        )

    def test_no_admin_url(self):
        thing = Something.objects.create()
        NotInAdmin.objects.create(id=1, fk=thing)
        body = render_to_string("related_objects_fk_test.html", {"obj": thing})
        self.assertEqual(body.strip(), "")

    def test_many_to_many(self):
        movie = Movie.objects.create(id=1, name="Yojimbo")
        actor = Actor.objects.create(name="Tatsuya Nakadai")
        actor.movies.add(movie)
        body = render_to_string("related_objects_m2m_test.html", {"obj": movie})
        self.assertEqual(
            body.strip(),
            '<a href="/adm/tests/actor/?movies__id__exact=1">Actors</a>',
        )

    def test_reverse_many_to_many(self):
        movie = Movie.objects.create(id=1, name="Yojimbo")
        actor = Actor.objects.create(name="Tatsuya Nakadai")
        actor.movies.add(movie)
        body = render_to_string("related_objects_m2m_test.html", {"obj": actor})
        self.assertEqual(body.strip(), "")

    def test_generic_foreign_key_not(self):
        book = Book.objects.create(name="Django")
        body = render_to_string("related_objects_generic_test.html", {"obj": book})
        self.assertEqual(body.strip(), "")

    def test_generic_foreign_key_present(self):
        book = Book.objects.create(name="Django")
        Image.objects.create(content_object=book)
        ct_pk = ContentType.objects.get_for_model(book).pk
        expected = (
            '<a href="/adm/tests/image/'
            f'?ct={ct_pk}&amp;obj_id__exact={book.pk}">Images</a>'
        )
        body = render_to_string(
            "related_objects_generic_test.html",
            {"obj": book},
        )
        self.assertEqual(body.strip(), expected)

    def test_generic_relation_and_gfk_present(self):
        journal = Journal.objects.create(name="Django")
        Image.objects.create(content_object=journal)
        ct_pk = ContentType.objects.get_for_model(journal).pk
        expected = (
            '<a href="/adm/tests/image/'
            f'?ct={ct_pk}&amp;obj_id__exact={journal.pk}">Images</a>'
        )
        body = render_to_string(
            "related_objects_generic_test.html",
            {"obj": journal},
        )
        self.assertEqual(body.strip(), expected)

    def test_with_removed_model(self):
        cache.clear()
        journal = Journal()
        ContentType.objects.create(model="gone", app_label="gone")
        body = render_to_string("related_objects_generic_test.html", {"obj": journal})
        self.assertEqual(body.strip(), "")


class RelativesAdminTests(TestCase):
    def setUp(self):
        self.user = User.objects.create_superuser("u", "u@example.com", "pass")

    def login(self):
        self.client.login(username=self.user.username, password="pass")

    def test_foreign_key_in_change_list(self):
        self.login()
        ship = Ship.objects.create(id=1, name="Star of India")
        Sailor.objects.create(name="John Ford", ship=ship)
        response = self.client.get(reverse("admin:tests_sailor_changelist"))
        self.assertIn(
            b'<a href="/adm/tests/ship/1/change/">Star of India</a>', response.content
        )

    def test_verbose_name_used_in_change_list(self):
        self.login()
        ship = Ship.objects.create(id=1, name="Star of India")
        Sailor.objects.create(name="John Ford", ship=ship)
        response = self.client.get(reverse("admin:tests_sailor_changelist"))
        self.assertIn(b"sea ship", response.content)  # verbose_name used



from django.urls import NoReverseMatch, reverse
from django.utils.encoding import smart_str
from django.utils.html import format_html


def get_admin_url(obj):
    """Return admin URL for given object (raise NoReverseMatch on error)"""
    return reverse(
        f"admin:{obj._meta.app_label}_{obj._meta.model_name}_change",
        args=[obj.pk],
    )


def object_edit_link(edit_text=None, blank_text=None):
    """
    Return function that takes an object and returns admin link to object

    Arguments:

    - ``edit_text`` is displayed in link text
    - ``blank_text`` is displayed in unlinked text (when no admin link)

    ``edit_text`` defaults to the object's unicode representation and
    ``blank_text`` defaults to the object's unicode representation if
    ``edit_text`` is None and an empty string otherwise
    """

    def object_link(obj):
        "Return admin link to given object or blank text if no link"
        link_text = smart_str(obj) if edit_text is None else edit_text
        try:
            if obj.pk:
                return format_html(
                    '<a href="{0}">{1}</a>',
                    get_admin_url(obj),
                    link_text,
                )
        except NoReverseMatch:
            pass
        if blank_text is None:
            if edit_text is None:
                return link_text
            else:
                return ""
        else:
            return blank_text

    object_link.__name__ = ""
    return object_link


object_link = object_edit_link()
object_link.__doc__ += "\n\nEquivalent to object_edit_link()(obj)"



from types import SimpleNamespace

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.cache import cache


class RelatedObject:
    """Generates fake django RelatedObject"""

    def __init__(self, field, ct_pk):
        self.field = SimpleNamespace(
            name=self.generate_field_name(field, ct_pk),
            model=field.model,
        )
        self.model = field.model
        self.name = self.generate_name(field)

    @staticmethod
    def generate_name(field):
        return ":".join([field.model._meta.app_label, field.model._meta.model_name])

    @staticmethod
    def generate_field_name(field, ct_pk):
        return "".join(map(str, [field.ct_field, "=", ct_pk, "&", field.fk_field]))


class GenericObjects:
    """
    Search GenericForeignKey over all models
    and returns related objects if has relations with given object.
    """

    def __init__(self, _object):
        self.obj = _object
        self.ct_pk = ContentType.objects.get_for_model(_object).pk
        self.cache_key = getattr(settings, "RELATIVES_CACHE_KEY", "relatives_cache")
        self.cache_time = getattr(settings, "RELATIVES_CACHE_TIME", 60 * 60 * 24)
        self.generic_objects = []

    def get_generic_objects(self):
        if not hasattr(self, "_generic_fields_cache"):
            self._fill_generic_fields_cache()
        for generic_field in self._generic_fields_cache:
            params = {
                generic_field.ct_field: self.ct_pk,
                generic_field.fk_field: self.obj.pk,
            }
            if generic_field.model.objects.filter(**params).exists():
                generic_object = RelatedObject(generic_field, self.ct_pk)
                self.generic_objects.append(generic_object)
        return self.generic_objects

    def _fill_generic_fields_cache(self):
        self._generic_fields_cache = cache.get(self.cache_key)
        if self._generic_fields_cache is not None:
            return
        self._generic_fields_cache = []
        for content_type in ContentType.objects.all():
            model_class = content_type.model_class()
            if not model_class:
                continue  # Model for content type doesn't exist anymore
            self._generic_fields_cache += [
                field
                for field in model_class._meta.private_fields
                if isinstance(field, GenericForeignKey)
            ]
        cache.set(self.cache_key, self._generic_fields_cache, self.cache_time)



from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.db import models


class Pirate(models.Model):
    """Pirates have no admin URL"""

    name = models.CharField(max_length=80)

    def __str__(self):
        return self.name


class Pet(models.Model):
    """Pet have an admin URL and link to Pirates"""

    owner = models.ForeignKey(Pirate, null=True, on_delete=models.SET_NULL)


class Ship(models.Model):
    """Ships have an admin URL and are linked to by sailors"""

    name = models.CharField(max_length=40)

    def __str__(self):
        return self.name


class Sailor(models.Model):
    """Sailors have an admin URL and sometimes link to ships"""

    name = models.CharField(max_length=80)
    ship = models.ForeignKey(
        Ship,
        null=True,
        on_delete=models.SET_NULL,
        verbose_name="sea ship",
    )

    @property
    def ship_name(self):
        return self.ship.name

    def __str__(self):
        return self.name


class Actor(models.Model):
    """Actors have an admin URL and a many-to-many relationship with movies"""

    name = models.CharField(max_length=80)
    movies = models.ManyToManyField("Movie", related_name="actors")


class Movie(models.Model):
    """Movies have an admin URL and are linked to (m2m) by actors"""

    name = models.CharField(max_length=80)


class Something(models.Model):
    """Somethings don't have an admin URL and are linked to by NotInAdmins"""

    text = models.CharField(max_length=10)


class NotInAdmin(models.Model):
    """NotInAdmins do not have an admin URL and link to Somethings"""

    fk = models.ForeignKey(Something, null=True, on_delete=models.SET_NULL)


class Shape(models.Model):
    """Shapes have a name and a lowercase name property."""

    name = models.CharField(max_length=10)

    @property
    def lower_name(self):
        return self.name.lower()


class Book(models.Model):
    """Book have an admin URL and are linked to images via GenericForeignKey"""

    name = models.CharField(max_length=10)


class Image(models.Model):
    """Image have an admin URL and link to Book via GenericForeignKey"""

    ct = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True)
    obj_id = models.PositiveIntegerField(null=True)
    content_object = GenericForeignKey("ct", "obj_id")


class Journal(models.Model):
    """Journal have an admin URL and are linked to images via GenericForeignKey
    also it have GenericRelation link to Images"""

    name = models.CharField(max_length=10)
    images = GenericRelation(Image)


class Eater(models.Model):
    """Eaters have an admin URL and Meal links to them in two ways."""

    name = models.CharField(max_length=10)


class Meal(models.Model):
    """Meals link to Eaters twice."""

    name = models.CharField(max_length=10)
    prepared = models.ForeignKey(
        Eater,
        related_name="meals_prepared",
        on_delete=models.CASCADE,
    )
    reviewed = models.ForeignKey(
        Eater,
        related_name="meals_reviewed",
        on_delete=models.CASCADE,
    )



from django.contrib.admin import ModelAdmin
from django.core.exceptions import FieldDoesNotExist

from .utils import object_link


def relation_link(related_field_name):
    def object_relation_link(original_obj):
        obj = getattr(original_obj, related_field_name)
        if obj is not None:
            return object_link(obj)

    object_relation_link.__name__ = related_field_name
    object_relation_link.admin_order_field = related_field_name
    return object_relation_link


def relation_link_from_field(related_field):
    object_relation_link = relation_link(related_field.name)
    object_relation_link.__name__ = related_field.verbose_name
    return object_relation_link


def related_link_or_attribute(model, attr):
    try:
        field = model._meta.get_field(attr)
    except FieldDoesNotExist:
        pass
    else:
        if field.is_relation:
            return relation_link_from_field(field)
    return attr


class RelativesMixin:
    """
    ``ModelAdmin`` mixin that makes list display foreign keys linked.

    This is used by ``RelativesAdmin``.
    This can be used along with Django's default ``ModelAdmin`` instead of inheriting from ``RelativesAdmin``.
    """

    def get_list_display(self, request):
        return [
            related_link_or_attribute(self.model, attr) for attr in self.list_display
        ]


class RelativesAdmin(RelativesMixin, ModelAdmin):
    """
    ``ModelAdmin`` that links to related fields.

    This automatically applies a variation of the ``object_link`` utility to all ``list_display`` fields.
    """



import contextlib
from importlib.metadata import PackageNotFoundError, version

from .model_admin import RelativesAdmin, RelativesMixin

with contextlib.suppress(PackageNotFoundError):  # pragma: nocover
    __version__ = version("django-relatives")


__all__ = ["RelativesMixin", "RelativesAdmin"]



from django.contrib import admin

from .. import RelativesAdmin
from . import models


class ShapeAdmin(admin.ModelAdmin):
    readonly_fields = ["name", "lower_name", "upper_name"]

    def upper_name(self, obj):
        return obj.name.upper()


admin.site.register(models.Shape, ShapeAdmin)


class ShipAdmin(admin.ModelAdmin):
    change_form_template = "relatives/change_form.html"
    readonly_fields = ["name"]


admin.site.register(models.Ship, ShipAdmin)


class SailorAdmin(RelativesAdmin):
    readonly_fields = ["ship"]
    list_display = ["name", "ship", "ship_name"]


admin.site.register(models.Sailor, SailorAdmin)


class PetAdmin(admin.ModelAdmin):
    readonly_fields = ["owner"]


admin.site.register(models.Pet, PetAdmin)


class ImageAdmin(admin.ModelAdmin):
    readonly_fields = ["content_object"]


admin.site.register(models.Image, ImageAdmin)


admin.site.register(models.Movie)
admin.site.register(models.Actor)
admin.site.register(models.Book)
admin.site.register(models.Journal)
admin.site.register(models.Eater)
admin.site.register(models.Meal)



from django.contrib.admin.utils import lookup_field
from django.core.exceptions import ObjectDoesNotExist
from django.template import Library
from django.urls import NoReverseMatch, reverse
from django.utils.encoding import smart_str
from django.utils.safestring import mark_safe

from ..template_helpers import GenericObjects
from ..utils import get_admin_url

register = Library()


@register.filter
def contents_or_fk_link(field):
    """
    Return field contents or link to related object if foreign key field

    Example Usage::

        {% load relatives %}
        {{ field|contents_or_fk_link }}
    """
    contents = field.contents()
    field_name = field.field["field"]
    obj = field.form.instance
    if not isinstance(field_name, str):
        return contents
    try:
        related_obj = getattr(obj, field_name)
    except (AttributeError, ObjectDoesNotExist):
        return contents
    else:
        model_field, _, _ = lookup_field(field_name, obj, field.model_admin)
        if (model_field is None or model_field.remote_field) and hasattr(
            related_obj, "_meta"
        ):
            try:
                return mark_safe(
                    f'<a href="{get_admin_url(related_obj)}">{contents}</a>'
                )
            except NoReverseMatch:
                pass
        return contents


@register.simple_tag
def related_objects(obj):
    """
    Return list of objects related to the given model instance

    Example Usage::

        {% load relatives %}
        {% related_objects obj as related_objects %}
        {% for related_obj in related_objects %}
            <a href="{{ related_obj.url }}">{{ related_obj.plural_name }}</a>
        {% endfor %}
    """
    object_list = []
    all_related_objects = [
        field
        for field in obj._meta.get_fields()
        if (field.one_to_many or field.one_to_one)
        and field.auto_created
        and not field.concrete
    ]
    all_related_m2m_objects = [
        field
        for field in obj._meta.get_fields(include_hidden=True)
        if field.many_to_many and field.auto_created
    ]
    related_objects = (
        all_related_objects
        + all_related_m2m_objects
        + GenericObjects(obj).get_generic_objects()
    )
    for related in related_objects:
        try:
            to_model = getattr(related, "related_model", related.model)
            url = reverse(
                f"admin:{to_model._meta.app_label}_{to_model._meta.model_name}_changelist"
            )
        except NoReverseMatch:
            continue
        if getattr(related, "related_name", None):
            plural_name = related.related_name.replace("_", " ")
        else:
            plural_name = to_model._meta.verbose_name_plural
        field = related.field.name
        if hasattr(related.field, "m2m_reverse_target_field_name"):
            field += "__" + related.field.m2m_reverse_target_field_name()
        elif hasattr(related.field, "target_field"):
            field += "__" + "".join(related.target_field.name)
        object_list.append(
            {
                "plural_name": plural_name,
                "url": smart_str(f"{url}?{field}__exact={obj.pk}"),
            }
        )
    return object_list



# flake8: noqa
# Generated by Django 3.0.2 on 2020-01-28 12:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Book",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="Journal",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="Movie",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=80)),
            ],
        ),
        migrations.CreateModel(
            name="Pirate",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=80)),
            ],
        ),
        migrations.CreateModel(
            name="Ship",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=40)),
            ],
        ),
        migrations.CreateModel(
            name="Something",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="Sailor",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=80)),
                (
                    "ship",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="tests.Ship",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Pet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="tests.Pirate",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="NotInAdmin",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "fk",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="tests.Something",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Image",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("obj_id", models.PositiveIntegerField(null=True)),
                (
                    "ct",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Actor",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=80)),
                (
                    "movies",
                    models.ManyToManyField(related_name="actors", to="tests.Movie"),
                ),
            ],
        ),
    ]



# Generated by Django 3.2.7 on 2021-09-27 16:19

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("tests", "0003_auto_20210927_1618"),
    ]

    operations = [
        migrations.CreateModel(
            name="Eater",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="Meal",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=10)),
                (
                    "prepared",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="meals_prepared",
                        to="tests.eater",
                    ),
                ),
                (
                    "reviewed",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="meals_reviewed",
                        to="tests.eater",
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 3.2.7 on 2021-09-27 16:18

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("tests", "0002_shape"),
    ]

    operations = [
        migrations.AlterField(
            model_name="actor",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="book",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="image",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="journal",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="movie",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="notinadmin",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="pet",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="pirate",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="sailor",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="shape",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="ship",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="something",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]
